# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: avp.proto
# Compatible with protobuf >= 4.21
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tavp.proto\x12\x03\x61vp\"\xa1\x03\n\x08Metadata\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x17\n\x0fsource_agent_id\x18\x02 \x01(\t\x12\x17\n\x0ftarget_agent_id\x18\x03 \x01(\t\x12\x10\n\x08model_id\x18\x04 \x01(\t\x12\x12\n\nhidden_dim\x18\x05 \x01(\r\x12\x12\n\nnum_layers\x18\x06 \x01(\r\x12&\n\x0cpayload_type\x18\x07 \x01(\x0e\x32\x10.avp.PayloadType\x12\x1c\n\x05\x64type\x18\x08 \x01(\x0e\x32\r.avp.DataType\x12\x14\n\x0ctensor_shape\x18\t \x03(\r\x12$\n\x04mode\x18\n \x01(\x0e\x32\x16.avp.CommunicationMode\x12\x18\n\x0b\x63ompression\x18\x0b \x01(\tH\x00\x88\x01\x01\x12\x12\n\navp_map_id\x18\r \x01(\t\x12\'\n\x05\x65xtra\x18\x0e \x03(\x0b\x32\x18.avp.Metadata.ExtraEntry\x1a,\n\nExtraEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0e\n\x0c_compression\"\xcc\x01\n\x10TransmitResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x14\n\x07message\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nrequest_id\x18\x03 \x01(\tH\x01\x88\x01\x01\x12/\n\x05\x65xtra\x18\x04 \x03(\x0b\x32 .avp.TransmitResponse.ExtraEntry\x1a,\n\nExtraEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\n\n\x08_messageB\r\n\x0b_request_id*<\n\x0bPayloadType\x12\x10\n\x0cHIDDEN_STATE\x10\x00\x12\x0c\n\x08KV_CACHE\x10\x01\x12\r\n\tEMBEDDING\x10\x02*.\n\x11\x43ommunicationMode\x12\n\n\x06LATENT\x10\x00\x12\r\n\tJSON_MODE\x10\x01*<\n\x08\x44\x61taType\x12\x0b\n\x07\x46LOAT32\x10\x00\x12\x0b\n\x07\x46LOAT16\x10\x01\x12\x0c\n\x08\x42\x46LOAT16\x10\x02\x12\x08\n\x04INT8\x10\x03\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'avp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_METADATA_EXTRAENTRY']._loaded_options = None
  _globals['_METADATA_EXTRAENTRY']._serialized_options = b'8\001'
  _globals['_TRANSMITRESPONSE_EXTRAENTRY']._loaded_options = None
  _globals['_TRANSMITRESPONSE_EXTRAENTRY']._serialized_options = b'8\001'
  _globals['_PAYLOADTYPE']._serialized_start=645
  _globals['_PAYLOADTYPE']._serialized_end=705
  _globals['_COMMUNICATIONMODE']._serialized_start=707
  _globals['_COMMUNICATIONMODE']._serialized_end=753
  _globals['_DATATYPE']._serialized_start=755
  _globals['_DATATYPE']._serialized_end=815
  _globals['_METADATA']._serialized_start=19
  _globals['_METADATA']._serialized_end=436
  _globals['_METADATA_EXTRAENTRY']._serialized_start=376
  _globals['_METADATA_EXTRAENTRY']._serialized_end=420
  _globals['_TRANSMITRESPONSE']._serialized_start=439
  _globals['_TRANSMITRESPONSE']._serialized_end=643
  _globals['_TRANSMITRESPONSE_EXTRAENTRY']._serialized_start=376
  _globals['_TRANSMITRESPONSE_EXTRAENTRY']._serialized_end=420
# @@protoc_insertion_point(module_scope)
